﻿using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace XnaDevRu.Assets
{
	internal class AssetDependencyWalker : Accessor
	{
		internal List<Dependency> DependencyList;

		public AssetDependencyWalker():base(null)
		{
			DependencyList = new List<Dependency>();
		}

		public override void VisitAsset<T>(ref T asset, string filedName)
		{
			if (asset == null)
				return;

			Asset curAsset = asset as Asset;

			BeginField(filedName);

			if (curAsset.Dependencies.Count > 0)
				DependencyList.Add(new Dependency(curAsset));

			curAsset.Accept(this);

			EndField(filedName);
		}

		public override void VisitSubAsset<T>(ref T asset, string filedName, Asset parent)
		{
			if (asset == null)
				return;

			BeginField(filedName);

			asset.Accept(this);

			EndField(filedName);
		}

	}

	internal class Dependency
	{
		public Asset Asset;

		public Dependency(Asset aset)
		{
			this.Asset = aset;
		}

		public string[] Dependencies
		{
			get { return Asset.DependencyList; }
		}

		public override string ToString()
		{
			return string.Format("{0}, dependencies count: {1}", Asset, Asset.Dependencies.Count);
		}
	}
}
