﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace XnaDevRu.Assets
{
    public class AssetFile
    {
        public string FileExtension;
        public string AssetName;
        public string AssetFileName;
        public string CachedAssetFileName;
        public string CachedAssetFileFullName;
        public string SourceAssetFileName;
        public string SourceAssetFileFullName;
        public string FolderName;
        public string FolderFullName;
        public DateTime CachedAssetVersion;
        public DateTime SourceAssetVersion;

        public AssetFile(AssetLibrary lib, string assetSource)
        {
            this.FileExtension = lib.FileExtension; // if the given file name has no extension we will use (*.asset)
            this.AssetName = Path.GetFileNameWithoutExtension(assetSource);

            if (Path.HasExtension(assetSource))
                this.FileExtension = Path.GetExtension(assetSource).ToLowerInvariant();

            this.AssetFileName = AssetName + FileExtension;

            this.SourceAssetFileName = this.AssetName + this.FileExtension;

            if (Path.HasExtension(assetSource))
                this.CachedAssetFileName = this.SourceAssetFileName + lib.FileExtension;
            else
                this.CachedAssetFileName = this.AssetFileName;

            this.FolderName = Path.GetDirectoryName(assetSource);

			if (!string.IsNullOrEmpty(this.FolderName))
			{
				this.FolderFullName = Path.GetDirectoryName(lib.GetFileFullPath(assetSource));
			}
			else
			{
				this.FolderFullName = Path.GetDirectoryName(lib.GetFileFullPath(SourceAssetFileName, true));

				if (this.FolderFullName.Length > lib.RootFolderPath.Length)
					this.FolderName = this.FolderFullName.Substring(lib.RootFolderPath.Length);
			}
			this.SourceAssetFileFullName = Path.Combine(FolderFullName, AssetFileName);
			this.CachedAssetFileFullName = Path.Combine(FolderFullName, CachedAssetFileName);

            this.CachedAssetVersion = lib.GetStreamVersion(CachedAssetFileFullName);
            this.SourceAssetVersion = lib.GetStreamVersion(SourceAssetFileFullName);

        }

        /// <summary>
        /// returns true if cached file exists and it is fresh enough
        /// </summary>
        public bool CacheFreshEnough
        {
            get
            {
                if (this.CachedAssetVersion == DateTime.MinValue)
                    return false; // cached file does not exists

                return (this.CachedAssetVersion >= this.SourceAssetVersion);
            }
        }

        public bool CacheExists
        {
            get
            {
                return (CachedAssetVersion > DateTime.MinValue);
            }
        }

        public bool SourceExists
        {
            get
            {
                return (SourceAssetVersion > DateTime.MinValue);
            }
        }

    }
}
