﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.Drawing.Text;

using XG = Microsoft.Xna.Framework.Graphics;

namespace XnaDevRu.Assets
{
	public class FontAsset : Asset<FontAsset>
	{
        internal byte[] charTable;
        internal short[] glyphPosXAarray;
        internal short[] glyphPosYAarray;
        internal byte[] glyphWidthAarray;
        internal byte[] glyphHeightAarray;
        internal byte[] textureData;
        internal int textureDataWidth;
        internal int textureDataHeight;
        internal string chars;


        private XG.Texture2D texture;

		public FontAsset()
		{
		}


        public override void Accept(Accessor visitor)
		{
            base.Accept(visitor);

            visitor.Visit(ref chars, "chars");
            //visitor.VisitArray(ref charTable, "CharTable");
            visitor.VisitArray(ref glyphPosXAarray, "GlyphPosXAarray");
            visitor.VisitArray(ref glyphPosYAarray, "GlyphPosYAarray");
            visitor.VisitArray(ref glyphWidthAarray, "GlyphWidthAarray");
            visitor.VisitArray(ref glyphHeightAarray, "GlyphHeightAarray");
            visitor.VisitArray(ref textureData, "TextureData");

            visitor.Visit(ref textureDataWidth, "TexDataWidth");
            visitor.Visit(ref textureDataHeight, "TexDataHeight");

            this.charTable = new byte[UInt16.MaxValue];

            int charIndex = 1;

            for (int i = 0; i < chars.Length; i++)
            {
                int charCode = (int)chars[i];

                this.charTable[charCode] = (byte)charIndex;

                charIndex++;

                if (charIndex > 254)
                    break;
            }
        }

        public XG.Texture2D Resolve(XG.GraphicsDevice device)
        {
            if (texture != null)
                return texture;

            if (textureData == null)
                return null;

            texture = new XG.Texture2D(device, textureDataWidth, textureDataHeight, 0, XG.ResourceUsage.None, XG.SurfaceFormat.Color, XG.ResourceManagementMode.Automatic);

            XG.Color[] colorData = new XG.Color[textureDataWidth * textureDataHeight];

            for (int x = 0; x < textureDataWidth; x++)
            {
                for (int y = 0; y < textureDataHeight; y++)
                {
                    byte c = (byte)textureData[y * textureDataWidth + x];
                    colorData[y * textureDataWidth + x] = new XG.Color(c, c, c, c);
                }
            }

            texture.SetData<XG.Color>(colorData);

            return texture;
        }

        public void SaveFontTexture(XG.GraphicsDevice device)
        {
            if (string.IsNullOrEmpty(Name))
                return;

            XG.Texture2D tex2D = Resolve(device);

            if (tex2D == null)
                return;

            tex2D.Save(Name + ".dds", XG.ImageFileFormat.Dds);
        }

        public short[] GlyphPosXAarray
        {
            get { return glyphPosXAarray; }
        }

        public short[] GlyphPosYAarray
        {
            get { return glyphPosYAarray; }
        }

        public byte[] GlyphWidthAarray
        {
            get { return glyphWidthAarray; }
        }

        public byte[] GlyphHeightAarray
        {
            get { return glyphHeightAarray; }
        }

        public byte[] CharTable
        {
            get { return charTable; }
        }

    }

//<FontName>tahoma</FontName> 
//... 
//<CharacterRegions> 
////английские стандартные 
//<CharacterRegion> 
//<Start>&#32;</Start> 
//<End>&#126;</End> 
//</CharacterRegion> 
////русские символы 
//<CharacterRegion> 
//<Start>&#1050;</Start> 
//<End>&#1200;</End> 
//</CharacterRegion> 
//...
//</CharacterRegions>

}
