﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace XnaDevRu.Assets
{
    public class TextureImporter : IAssetImporter
    {
        AssetLibrary _library;
        private bool _useEmbedding;

        public TextureImporter(bool embedding)
        {
            _useEmbedding = embedding;
        }

        #region IAssetImporter Members


        public T Load<T>(AssetFile source) where T : Asset, new()
        {
            if (typeof(T) != typeof(TextureAsset))
                return null;

            TextureAsset result = null;

            if (!_useEmbedding) // create texture asset as external file refernce
            {
                result = new TextureAsset(source.SourceAssetFileFullName, TextureAsset.Type.Texture2D);
            }
            else
            {
                Stream stream = Library.OpenReadStream(source.SourceAssetFileFullName);

                if (stream == null)
                    return null;

                result = new TextureAsset(stream, TextureAsset.Type.Texture2D);

                stream.Close();
                stream.Dispose();
            }

            result.Name = source.AssetName;
            result.Version = source.SourceAssetVersion;

            return result as T;

        }

        public void ResetImporter()
        {
        }

        public string FileExtension
        {
            get { return ".dds;.jpg;.png;.bmp;.tga"; }
        }

        public AssetLibrary Library
        {
            get
            {
                return _library;
            }
            set
            {
                _library = value;
            }
        }

        public bool HasParentImporter
        {
            get { return true; }
        }

        #endregion
    }
}
