using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using XnaDevRu.Engine.Threading;

namespace XnaDevRu.Engine
{
    public class GameEngine : IDisposable
    {
        #region GameManager

        class GameManager : Game
        {
            GameEngine engine;

            public GameManager(GameEngine eng)
            {
                engine = eng;
            }

            protected override void Initialize()
            {
                engine.Init();
            }

            protected override void Draw(GameTime gameTime)
            {
                engine.Tick(gameTime);
            }
        }

        #endregion

        GameManager game;
        GraphicsDeviceManager graphics;
        IThreadsManager threadsManager;

        protected IThreadsManager ThreadsManager
        {
            get { return threadsManager; }
        }

        protected GraphicsDeviceManager Graphics
        {
            get { return graphics; }
        }

        public GameEngine()
        {
            game = new GameManager(this);
            graphics = new GraphicsDeviceManager(game);
            threadsManager = new ThreadsManager();
        }

        protected virtual void Init()
        {

        }

        void Tick(GameTime gameTime)
        {
            if (!ThreadsManager.IsRunning)
                ThreadsManager.StartTimers();

            threadsManager.Tick();
        }

        public virtual void Run()
        {
            game.Run();
        }

        public void Dispose()
        {
            threadsManager.Dispose();
            threadsManager = null;

            game.Dispose();
            game = null;
        }
    }
}
