using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using XnaDevRu.Physics;

namespace Physics.Test
{
    /// <summary>
    /// A drawable box object that can be attached to an XPA Solid.
    /// </summary>
    public class BoxEntity : Entity
    {
        VertexPositionTexture[] verts;

        public BoxEntity(Simulator s, Vector3 dim, Texture2D texture)
            : base(s, texture)
        {
            verts = CreateBox(dim);
            mSolid = s.CreateSolid();

            BoxShapeData boxShape = new BoxShapeData();
            boxShape.Dimensions = dim;
            mSolid.AddShape(boxShape);

        }

        static VertexPositionTexture Create(Vector3 s, float x, float y, float z, float tx, float ty)
        {
            Vector3 v = new Vector3(s.X * x * 0.5f, s.Y * y * 0.5f, s.Z * z * 0.5f);
            Vector2 t = new Vector2(tx, ty);

            return new VertexPositionTexture(v, t);
        }

        static VertexPositionTexture[] CreateBox(Vector3 scale)
        {

            VertexPositionTexture[] verts = new VertexPositionTexture[36];

            // Front face
            verts[0] = Create(scale, -1.0f, 1.0f, 1.0f, 0.0f, 0.0f);
            verts[1] = Create(scale, -1.0f, -1.0f, 1.0f, 0.0f, 1.0f);
            verts[2] = Create(scale, 1.0f, 1.0f, 1.0f, 1.0f, 0.0f);
            verts[3] = Create(scale, -1.0f, -1.0f, 1.0f, 0.0f, 1.0f);
            verts[4] = Create(scale, 1.0f, -1.0f, 1.0f, 1.0f, 1.0f);
            verts[5] = Create(scale, 1.0f, 1.0f, 1.0f, 1.0f, 0.0f);

            // Back face (remember this is facing *away* from the camera, so vertices should be clockwise order)
            verts[6] = Create(scale, -1.0f, 1.0f, -1.0f, 0.0f, 0.0f);
            verts[7] = Create(scale, 1.0f, 1.0f, -1.0f, 1.0f, 0.0f);
            verts[8] = Create(scale, -1.0f, -1.0f, -1.0f, 0.0f, 1.0f);
            verts[9] = Create(scale, -1.0f, -1.0f, -1.0f, 0.0f, 1.0f);
            verts[10] = Create(scale, 1.0f, 1.0f, -1.0f, 1.0f, 0.0f);
            verts[11] = Create(scale, 1.0f, -1.0f, -1.0f, 1.0f, 1.0f);

            // Top face
            verts[12] = Create(scale, -1.0f, 1.0f, 1.0f, 0.0f, 0.0f);
            verts[13] = Create(scale, 1.0f, 1.0f, -1.0f, 1.0f, 1.0f);
            verts[14] = Create(scale, -1.0f, 1.0f, -1.0f, 0.0f, 1.0f);
            verts[15] = Create(scale, -1.0f, 1.0f, 1.0f, 0.0f, 0.0f);
            verts[16] = Create(scale, 1.0f, 1.0f, 1.0f, 1.0f, 0.0f);
            verts[17] = Create(scale, 1.0f, 1.0f, -1.0f, 1.0f, 1.0f);

            // Bottom face (remember this is facing *away* from the camera, so vertices should be clockwise order)
            verts[18] = Create(scale, -1.0f, -1.0f, 1.0f, 0.0f, 0.0f);
            verts[19] = Create(scale, -1.0f, -1.0f, -1.0f, 0.0f, 1.0f);
            verts[20] = Create(scale, 1.0f, -1.0f, -1.0f, 1.0f, 1.0f);
            verts[21] = Create(scale, -1.0f, -1.0f, 1.0f, 0.0f, 0.0f);
            verts[22] = Create(scale, 1.0f, -1.0f, -1.0f, 1.0f, 1.0f);
            verts[23] = Create(scale, 1.0f, -1.0f, 1.0f, 1.0f, 0.0f);

            // Left face
            verts[24] = Create(scale, -1.0f, 1.0f, 1.0f, 0.0f, 0.0f);
            verts[25] = Create(scale, -1.0f, -1.0f, -1.0f, 1.0f, 1.0f);
            verts[26] = Create(scale, -1.0f, -1.0f, 1.0f, 1.0f, 0.0f);
            verts[27] = Create(scale, -1.0f, 1.0f, -1.0f, 0.0f, 1.0f);
            verts[28] = Create(scale, -1.0f, -1.0f, -1.0f, 1.0f, 1.0f);
            verts[29] = Create(scale, -1.0f, 1.0f, 1.0f, 0.0f, 0.0f);

            // Right face (remember this is facing *away* from the camera, so vertices should be clockwise order)
            verts[30] = Create(scale, 1.0f, 1.0f, 1.0f, 0.0f, 0.0f);
            verts[31] = Create(scale, 1.0f, -1.0f, 1.0f, 1.0f, 0.0f);
            verts[32] = Create(scale, 1.0f, -1.0f, -1.0f, 1.0f, 1.0f);
            verts[33] = Create(scale, 1.0f, 1.0f, -1.0f, 0.0f, 1.0f);
            verts[34] = Create(scale, 1.0f, 1.0f, 1.0f, 0.0f, 0.0f);
            verts[35] = Create(scale, 1.0f, -1.0f, -1.0f, 1.0f, 1.0f);

            return verts;
        }


        public override void Draw(GraphicsDeviceManager gfx)
        {
            gfx.GraphicsDevice.DrawUserPrimitives<VertexPositionTexture>(PrimitiveType.TriangleList, verts, 0, 12);

        }

        public override void Destory()
        {
            base.Destory();

            this.verts = null;
        }
    }
}
