using System;
using System.Collections.Generic;
using System.Text;

using XnaDevRu.Physics;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Physics.Test
{
    /// <summary>
    /// A drawable object that can be attached to an XPA Solid.
    /// </summary>
    public abstract class Entity
    {
        public Matrix WorldMatrix;
        protected Solid mSolid;
        protected Simulator sim;
        public Texture2D Texture;

        public Entity(Simulator s, Texture2D tex)
        {
            sim = s;
            Texture = tex;
        }

        public virtual void Destory()
        {
            sim.DestroySolid(mSolid);
        }

        /// <summary>
        /// Updates world matrix for this entity.
        /// Should be called before draw()
        /// </summary>
        public void Update()
        {
            if (mSolid == null)
                return;

            WorldMatrix = mSolid.Transform;
            //WorldMatrix.Translation = mSolid.Position;
        }

        /// <summary>
        /// Draws the object.  Does nothing until after being attached to a Solid.
        /// </summary>
        public abstract void Draw(GraphicsDeviceManager gfx);

        /// <summary>
        /// Attaches the Entity to an XPA Solid.
        /// </summary>
        /// <param name="solid"></param>
		public void AttachToSolid(Solid solid )
		{
			mSolid = solid;
		}

        public virtual bool Static
        {
            set
            {
                if (mSolid == null)
                    return;

                mSolid.Static = value;
            }
        }

        public Vector3 Position
        {
            get
            {
                return WorldMatrix.Translation;
            }
            set
            {

                if (mSolid == null)
                    return;

                WorldMatrix.Translation = value;
                mSolid.Position = value;
            }
        }

        public Solid Solid
        {
            get { return this.mSolid; }
        }
    }
}
