using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace Physics.Test
{
    public partial class fpsComponent : DrawableGameComponent, IDrawable
    {
        public enum ShowMethodEnum
        {
            TitleBar,
            Screen,
            Both
        }

        public fpsComponent(Game game):base(game)
        {
            InitializeComponent();
        }

        public string Text;
        
        private ShowMethodEnum showMethod;

        public ShowMethodEnum ShowMethod
        {
            get { return showMethod; }
            set { showMethod = value; }
        }

        private float fps;
        public float FPS
        {
            get { return fps; }
        }

        private float updateInterval=1.0f;
        public float UpdateInterval
        {
            get { return updateInterval; }
            set { updateInterval = value; }
        }
        float timeSinceLastUpdate = 0.0f;
        float framecount = 0;


	
        BitmapFont font;

        public override void Initialize()
        {
            IGraphicsDeviceService graphicsDeviceService = (IGraphicsDeviceService)Game.Services.GetService(typeof(IGraphicsDeviceService));
            graphicsDeviceService.DeviceReset += new EventHandler(GraphicsDevice_DeviceReset);
            font = new BitmapFont("Fonts\\calibri.xml");
            LoadResources(graphicsDeviceService.GraphicsDevice);
            
        }

        void GraphicsDevice_DeviceReset(object sender, EventArgs e)
        {
            LoadResources((GraphicsDeviceManager)sender);
        }

        void LoadResources(GraphicsDeviceManager component)
        {
            LoadResources(component.GraphicsDevice);
        }
        void LoadResources(GraphicsDevice device)
        {
            font.Reset(device);
        }
        public override void Update(GameTime gametime)
        {
            // TODO: Add your update code here
            float elapsed = (float)gametime.ElapsedGameTime.TotalSeconds;
            framecount++;
            timeSinceLastUpdate += elapsed;
            if (timeSinceLastUpdate > updateInterval)
            {
                //fps = 1 / elapsed;
                fps = framecount / timeSinceLastUpdate; //mean fps over updateIntrval
                framecount = 0;
                timeSinceLastUpdate -= updateInterval;
            }

        }

        public override void  Draw(GameTime gameTime)
        {
            string f = string.Format("fps : {0:F2}", fps) + Text; 
            if (showMethod == ShowMethodEnum.Both)
            {
                Game.Window.Title = "FPS : " + fps.ToString();
                font.DrawString(10, 5, Color.Gold, f);
            }
            else
            {
                if (showMethod == ShowMethodEnum.Screen)
                {
                    font.DrawString(10, 5, Color.Gold, "{0}", f);
                    //font.DrawString(102, 102, Color.Black, "New Game");
                    //font.DrawString(100, 100, Color.GreenYellow, "New Game");
                }
                else
                    Game.Window.Title = "FPS : " + fps.ToString();
            }
        }
    }
}